module Wrapybara
	module Element
		def get_element(xpath, scope)
			if scope
				# if scope starts with '/html', let's assume (for now) that it's already an xpath
				unless scope =~ /^\/html/
					# if there are multiple scopes (ie, "div1 div2 para1"), the following makes 
					# xpath look like "id('div1')/*[@id='div2']/*[@id='para1']"

					scope = scope.split(' ')
					id = scope.shift
					scope = "id('#{id}')" + scope.inject('') { |output, s| output += "//*[@id='#{s}']" }
				end

				# xpath might come in like //something, and since we would be looking for an element
				# in the scope of another element, we need to make sure we aren't looking at the whole
				# document by adding the dot at the beginning
				xpath = ".#{xpath}" if xpath[0..1] == '//'
				Capybara.find(scope).find(xpath) rescue nil
			else
				Capybara.find(xpath) rescue nil
			end
		end
		
		def element
			@element
		end
		
		def path
			element.path
		end
		
		def style
			element[:style] || ''
		end

		def click
			element.click
		end
		
		def exists?
			!@element.nil?
		end

		def should_exist(message = nil)
			message ||= "Expected element #{self.element_identifier} to exist"
			raise UnmetExpectation, message unless self.exists?
		end

		def should_not_exist(message = nil)
			message ||= "Did not expect element #{self.element_identifier} to exist"
			raise UnmetExpectation, message if self.exists?
		end
		
		def should_have_attribute(attribute, value = nil)
			the_attribute = Attribute.new(self, attribute)
			the_attribute.should_exist
			the_attribute.value.should_be value if value
		end
		
		def should_not_have_attribute(attribute, value = nil)
			the_attribute = Attribute.new(self, attribute)
			if value
				the_attribute.should_exist
				the_attribute.value.should_not_be value
			else
				the_attribute.should_not_exist
			end
		end
		
		def enabled?
			!disabled?
		end
		
		def disabled?
			self.should_exist
			['true', 'disabled'].include?(@element[:disabled])
		end
		
		def should_be_enabled
			message = "Expected element #{self.element_identifier} to be enabled"
			raise UnmetExpectation, message unless self.enabled?
		end
		
		def should_not_be_enabled
			message = "Did not expect element #{self.element_identifier} to be enabled"
			raise UnmetExpectation, message if self.enabled?
		end
		
		def should_be_disabled
			message = "Expected element #{self.element_identifier} to be disabled"
			raise UnmetExpectation, message unless self.disabled?
		end
		
		def should_not_be_disabled
			message = "Did not expect element #{self.element_identifier} to be disabled"
			raise UnmetExpectation, message if self.disabled?
		end
		
		def visible?
			self.should_exist
			
			# determining visibility of something is challenging, to say the least. this approach is to start with the subject element
			# and check its style attribute for display: none or visibility: hidden. if neither is found, work backward up the
			# object hierarchy checking each parent.
			
			return false if self.style.downcase.gsub(' ', '') =~ /(display:none|visibility:hidden)/

			parent = get_element(self.path + '/..', nil)
			while parent && parent.path != '/html'
				style = parent[:style] || ''
				return false if style.downcase.gsub(' ', '') =~ /(display:none|visibility:hidden)/
				parent = get_element(parent.path + '/..', nil)
			end
			
			return true
		end
		
		def should_be_visible
			message = "Expected content #{self.element_identifier} to be visible"
			raise UnmetExpectation, message unless self.visible?
		end
		
		def should_not_be_visible
			message = "Did not expect content #{self.element_identifier} to be visible"
			raise UnmetExpectation, message if self.visible?
		end
		
		def within(scope)
			scope ? " within '#{scope}'" : ''
		end
		
		def element_identifier
			"#{@how} '#{@identifier}'#{self.within(@scope)}"
		end
		
		def parent_identifier
			return '' unless @parent
			"#{@parent.how} '#{@parent.identifier}'#{self.within(@parent.scope)}"
		end
		
		def default_scope
			nil
		end
		
		def default_how
			'labeled'
		end
	end
end